==> nukleus-oauth-0.56-sources.jar: org/reaktivity/nukleus/oauth/internal/types/stream/WindowFW.java <==
// TODO: license
package org.reaktivity.nukleus.oauth.internal.types.stream;

import java.util.function.Consumer;
import org.agrona.BitUtil;
import org.agrona.DirectBuffer;
import org.agrona.MutableDirectBuffer;
import org.reaktivity.nukleus.oauth.internal.types.Flyweight;
import org.reaktivity.nukleus.oauth.internal.types.OctetsFW;

public final class WindowFW extends Flyweight {
  public static final int FIELD_OFFSET_ROUTE_ID = 0;

  private static final int FIELD_SIZE_ROUTE_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_STREAM_ID = FIELD_OFFSET_ROUTE_ID + FIELD_SIZE_ROUTE_ID;

  private static final int FIELD_SIZE_STREAM_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_TIMESTAMP = FIELD_OFFSET_STREAM_ID + FIELD_SIZE_STREAM_ID;

  private static final int FIELD_SIZE_TIMESTAMP = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_TRACE = FIELD_OFFSET_TIMESTAMP + FIELD_SIZE_TIMESTAMP;

  private static final int FIELD_SIZE_TRACE = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_AUTHORIZATION = FIELD_OFFSET_TRACE + FIELD_SIZE_TRACE;

  private static final int FIELD_SIZE_AUTHORIZATION = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_CREDIT = FIELD_OFFSET_AUTHORIZATION + FIELD_SIZE_AUTHORIZATION;

  private static final int FIELD_SIZE_CREDIT = BitUtil.SIZE_OF_INT;

  public static final int FIELD_OFFSET_PADDING = FIELD_OFFSET_CREDIT + FIELD_SIZE_CREDIT;

  private static final int FIELD_SIZE_PADDING = BitUtil.SIZE_OF_INT;

  public static final int FIELD_OFFSET_GROUP_ID = FIELD_OFFSET_PADDING + FIELD_SIZE_PADDING;

  private static final int FIELD_SIZE_GROUP_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_CAPABILITIES = FIELD_OFFSET_GROUP_ID + FIELD_SIZE_GROUP_ID;

  private static final int FIELD_SIZE_CAPABILITIES = BitUtil.SIZE_OF_BYTE;

  public static final int FIELD_OFFSET_EXTENSION = FIELD_OFFSET_CAPABILITIES + FIELD_SIZE_CAPABILITIES;

  public static final int TYPE_ID = 0x40000002;

  private final OctetsFW extensionRO = new OctetsFW();

  public long routeId() {
    return buffer().getLong(offset() + FIELD_OFFSET_ROUTE_ID);
  }

  public long streamId() {
    return buffer().getLong(offset() + FIELD_OFFSET_STREAM_ID);
  }

  public long timestamp() {
    return buffer().getLong(offset() + FIELD_OFFSET_TIMESTAMP);
  }

  public long trace() {
    return buffer().getLong(offset() + FIELD_OFFSET_TRACE);
  }

  public long authorization() {
    return buffer().getLong(offset() + FIELD_OFFSET_AUTHORIZATION);
  }

  public int credit() {
    return buffer().getInt(offset() + FIELD_OFFSET_CREDIT);
  }

  public int padding() {
    return buffer().getInt(offset() + FIELD_OFFSET_PADDING);
  }

  public long groupId() {
    return buffer().getLong(offset() + FIELD_OFFSET_GROUP_ID);
  }

  public int capabilities() {
    return (int)(buffer().getByte(offset() + FIELD_OFFSET_CAPABILITIES) & 0xFF);
  }

  public OctetsFW extension() {
    return extensionRO;
  }

  public int typeId() {
    return TYPE_ID;
  }

  @Override
  public WindowFW wrap(DirectBuffer buffer, int offset, int maxLimit) {
    super.wrap(buffer, offset, maxLimit);
    extensionRO.wrap(buffer, offset + FIELD_OFFSET_EXTENSION, maxLimit);
    checkLimit(limit(), maxLimit);
    return this;
  }

  @Override
  public WindowFW tryWrap(DirectBuffer buffer, int offset, int maxLimit) {
    if (null == super.tryWrap(buffer, offset, maxLimit)) {
      return null;
    }
    if (null == extensionRO.tryWrap(buffer, offset + FIELD_OFFSET_EXTENSION, maxLimit)) {
      return null;
    }
    if (limit() > maxLimit) {
      return null;
    }
    return this;
  }

  @Override
  public int limit() {
    return extensionRO.limit();
  }

  @Override
  public String toString() {
    return String.format("WINDOW [routeId=%d, streamId=%d, timestamp=%d, trace=%d, authorization=%d, credit=%d, padding=%d, groupId=%d, capabilities=%d, extension=%s]", routeId(), streamId(), timestamp(), trace(), authorization(), credit(), padding(), groupId(), capabilities(), extension());
  }

  public static final class Builder extends Flyweight.Builder<WindowFW> {
    private static final int INDEX_ROUTE_ID = 0;

    private static final int INDEX_STREAM_ID = 1;

    private static final int INDEX_TIMESTAMP = 2;

    private static final long DEFAULT_TIMESTAMP = 0;

    private static final int INDEX_TRACE = 3;

    private static final long DEFAULT_TRACE = 0;

    private static final int INDEX_AUTHORIZATION = 4;

    private static final long DEFAULT_AUTHORIZATION = 0;

    private static final int INDEX_CREDIT = 5;

    private static final int INDEX_PADDING = 6;

    private static final int INDEX_GROUP_ID = 7;

    private static final int INDEX_CAPABILITIES = 8;

    private static final int DEFAULT_CAPABILITIES = 0;

    private static final int INDEX_EXTENSION = 9;

    private static final int FIELD_COUNT = 10;

    private final OctetsFW.Builder extensionRW = new OctetsFW.Builder();

    private int lastFieldSet = -1;

    public Builder() {
      super(new WindowFW());
    }

    public Builder routeId(long value) {
      assert lastFieldSet == INDEX_ROUTE_ID - 1;
      int newLimit = limit() + FIELD_SIZE_ROUTE_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_ROUTE_ID;
      limit(newLimit);
      return this;
    }

    public Builder streamId(long value) {
      assert lastFieldSet == INDEX_STREAM_ID - 1;
      int newLimit = limit() + FIELD_SIZE_STREAM_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_STREAM_ID;
      limit(newLimit);
      return this;
    }

    public Builder timestamp(long value) {
      assert lastFieldSet == INDEX_TIMESTAMP - 1;
      int newLimit = limit() + FIELD_SIZE_TIMESTAMP;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_TIMESTAMP;
      limit(newLimit);
      return this;
    }

    public Builder trace(long value) {
      if (lastFieldSet < INDEX_TIMESTAMP) {
        timestamp(DEFAULT_TIMESTAMP);
      }
      assert lastFieldSet == INDEX_TRACE - 1;
      int newLimit = limit() + FIELD_SIZE_TRACE;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_TRACE;
      limit(newLimit);
      return this;
    }

    public Builder authorization(long value) {
      if (lastFieldSet < INDEX_TRACE) {
        trace(DEFAULT_TRACE);
      }
      assert lastFieldSet == INDEX_AUTHORIZATION - 1;
      int newLimit = limit() + FIELD_SIZE_AUTHORIZATION;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_AUTHORIZATION;
      limit(newLimit);
      return this;
    }

    public Builder credit(int value) {
      if (lastFieldSet < INDEX_AUTHORIZATION) {
        authorization(DEFAULT_AUTHORIZATION);
      }
      assert lastFieldSet == INDEX_CREDIT - 1;
      int newLimit = limit() + FIELD_SIZE_CREDIT;
      checkLimit(newLimit, maxLimit());
      buffer().putInt(limit(), value);
      lastFieldSet = INDEX_CREDIT;
      limit(newLimit);
      return this;
    }

    public Builder padding(int value) {
      assert lastFieldSet == INDEX_PADDING - 1;
      int newLimit = limit() + FIELD_SIZE_PADDING;
      checkLimit(newLimit, maxLimit());
      buffer().putInt(limit(), value);
      lastFieldSet = INDEX_PADDING;
      limit(newLimit);
      return this;
    }

    public Builder groupId(long value) {
      assert lastFieldSet == INDEX_GROUP_ID - 1;
      int newLimit = limit() + FIELD_SIZE_GROUP_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_GROUP_ID;
      limit(newLimit);
      return this;
    }

    public Builder capabilities(int value) {
      if (value < 0) {
        throw new IllegalArgumentException(String.format("Value %d too low for field \"capabilities\"", value));
      }
      if (value > 0XFF) {
        throw new IllegalArgumentException(String.format("Value %d too high for field \"capabilities\"", value));
      }
      assert lastFieldSet == INDEX_CAPABILITIES - 1;
      int newLimit = limit() + FIELD_SIZE_CAPABILITIES;
      checkLimit(newLimit, maxLimit());
      buffer().putByte(limit(), (byte)(value & 0xFF));
      lastFieldSet = INDEX_CAPABILITIES;
      limit(newLimit);
      return this;
    }

    private OctetsFW.Builder extension() {
      if (lastFieldSet < INDEX_CAPABILITIES) {
        capabilities(DEFAULT_CAPABILITIES);
      }
      assert lastFieldSet == INDEX_EXTENSION - 1;
      return extensionRW.wrap(buffer(), limit(), maxLimit());
    }

    public Builder extension(OctetsFW value) {
      OctetsFW.Builder extensionRW = extension();
      extensionRW.set(value);
      limit(extensionRW.build().limit());
      lastFieldSet = INDEX_EXTENSION;
      return this;
    }

    public Builder extension(Consumer<OctetsFW.Builder> mutator) {
      OctetsFW.Builder extensionRW = extension();
      mutator.accept(extensionRW);
      limit(extensionRW.build().limit());
      lastFieldSet = INDEX_EXTENSION;
      return this;
    }

    public Builder extension(DirectBuffer buffer, int offset, int length) {
      OctetsFW.Builder extensionRW = extension();
      extensionRW.set(buffer, offset, length);
      limit(extensionRW.build().limit());
      lastFieldSet = INDEX_EXTENSION;
      return this;
    }

    @Override
    public Builder wrap(MutableDirectBuffer buffer, int offset, int maxLimit) {
      super.wrap(buffer, offset, maxLimit);
      lastFieldSet = -1;
      super.wrap(buffer, offset, maxLimit);
      limit(offset);
      return this;
    }

    @Override
    public Builder rewrap() {
      super.rewrap();
      return this;
    }

    @Override
    public WindowFW build() {
      if (lastFieldSet < INDEX_EXTENSION) {
        extension(b -> { });
      }
      assert lastFieldSet == FIELD_COUNT - 1;
      lastFieldSet = -1;
      return super.build();
    }
  }
}

==> nukleus-oauth-0.58-sources.jar: org/reaktivity/nukleus/oauth/internal/types/stream/WindowFW.java <==
// TODO: license
package org.reaktivity.nukleus.oauth.internal.types.stream;

import org.agrona.BitUtil;
import org.agrona.DirectBuffer;
import org.agrona.MutableDirectBuffer;
import org.reaktivity.nukleus.oauth.internal.types.Flyweight;

public final class WindowFW extends Flyweight {
  public static final int FIELD_OFFSET_ROUTE_ID = 0;

  private static final int FIELD_SIZE_ROUTE_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_STREAM_ID = FIELD_OFFSET_ROUTE_ID + FIELD_SIZE_ROUTE_ID;

  private static final int FIELD_SIZE_STREAM_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_TIMESTAMP = FIELD_OFFSET_STREAM_ID + FIELD_SIZE_STREAM_ID;

  private static final int FIELD_SIZE_TIMESTAMP = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_TRACE_ID = FIELD_OFFSET_TIMESTAMP + FIELD_SIZE_TIMESTAMP;

  private static final int FIELD_SIZE_TRACE_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_AUTHORIZATION = FIELD_OFFSET_TRACE_ID + FIELD_SIZE_TRACE_ID;

  private static final int FIELD_SIZE_AUTHORIZATION = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_BUDGET_ID = FIELD_OFFSET_AUTHORIZATION + FIELD_SIZE_AUTHORIZATION;

  private static final int FIELD_SIZE_BUDGET_ID = BitUtil.SIZE_OF_LONG;

  public static final int FIELD_OFFSET_CREDIT = FIELD_OFFSET_BUDGET_ID + FIELD_SIZE_BUDGET_ID;

  private static final int FIELD_SIZE_CREDIT = BitUtil.SIZE_OF_INT;

  public static final int FIELD_OFFSET_PADDING = FIELD_OFFSET_CREDIT + FIELD_SIZE_CREDIT;

  private static final int FIELD_SIZE_PADDING = BitUtil.SIZE_OF_INT;

  public static final int FIELD_OFFSET_CAPABILITIES = FIELD_OFFSET_PADDING + FIELD_SIZE_PADDING;

  private static final int FIELD_SIZE_CAPABILITIES = BitUtil.SIZE_OF_BYTE;

  public static final int TYPE_ID = 0x40000002;

  public long routeId() {
    return buffer().getLong(offset() + FIELD_OFFSET_ROUTE_ID);
  }

  public long streamId() {
    return buffer().getLong(offset() + FIELD_OFFSET_STREAM_ID);
  }

  public long timestamp() {
    return buffer().getLong(offset() + FIELD_OFFSET_TIMESTAMP);
  }

  public long traceId() {
    return buffer().getLong(offset() + FIELD_OFFSET_TRACE_ID);
  }

  public long authorization() {
    return buffer().getLong(offset() + FIELD_OFFSET_AUTHORIZATION);
  }

  public long budgetId() {
    return buffer().getLong(offset() + FIELD_OFFSET_BUDGET_ID);
  }

  public int credit() {
    return buffer().getInt(offset() + FIELD_OFFSET_CREDIT);
  }

  public int padding() {
    return buffer().getInt(offset() + FIELD_OFFSET_PADDING);
  }

  public int capabilities() {
    return (int)(buffer().getByte(offset() + FIELD_OFFSET_CAPABILITIES) & 0xFF);
  }

  public int typeId() {
    return TYPE_ID;
  }

  @Override
  public WindowFW wrap(DirectBuffer buffer, int offset, int maxLimit) {
    super.wrap(buffer, offset, maxLimit);
    checkLimit(limit(), maxLimit);
    return this;
  }

  @Override
  public WindowFW tryWrap(DirectBuffer buffer, int offset, int maxLimit) {
    if (null == super.tryWrap(buffer, offset, maxLimit)) {
      return null;
    }
    if (limit() > maxLimit) {
      return null;
    }
    return this;
  }

  @Override
  public int limit() {
    return offset() + FIELD_OFFSET_CAPABILITIES + FIELD_SIZE_CAPABILITIES;
  }

  @Override
  public String toString() {
    return String.format("WINDOW [routeId=%d, streamId=%d, timestamp=%d, traceId=%d, authorization=%d, budgetId=%d, credit=%d, padding=%d, capabilities=%d]", routeId(), streamId(), timestamp(), traceId(), authorization(), budgetId(), credit(), padding(), capabilities());
  }

  public static final class Builder extends Flyweight.Builder<WindowFW> {
    private static final int INDEX_ROUTE_ID = 0;

    private static final int INDEX_STREAM_ID = 1;

    private static final int INDEX_TIMESTAMP = 2;

    private static final long DEFAULT_TIMESTAMP = 0;

    private static final int INDEX_TRACE_ID = 3;

    private static final long DEFAULT_TRACE_ID = 0;

    private static final int INDEX_AUTHORIZATION = 4;

    private static final long DEFAULT_AUTHORIZATION = 0;

    private static final int INDEX_BUDGET_ID = 5;

    private static final int INDEX_CREDIT = 6;

    private static final int INDEX_PADDING = 7;

    private static final int INDEX_CAPABILITIES = 8;

    private static final int DEFAULT_CAPABILITIES = 0;

    private static final int FIELD_COUNT = 9;

    private int lastFieldSet = -1;

    public Builder() {
      super(new WindowFW());
    }

    public Builder routeId(long value) {
      assert lastFieldSet == INDEX_ROUTE_ID - 1;
      int newLimit = limit() + FIELD_SIZE_ROUTE_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_ROUTE_ID;
      limit(newLimit);
      return this;
    }

    public Builder streamId(long value) {
      assert lastFieldSet == INDEX_STREAM_ID - 1;
      int newLimit = limit() + FIELD_SIZE_STREAM_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_STREAM_ID;
      limit(newLimit);
      return this;
    }

    public Builder timestamp(long value) {
      assert lastFieldSet == INDEX_TIMESTAMP - 1;
      int newLimit = limit() + FIELD_SIZE_TIMESTAMP;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_TIMESTAMP;
      limit(newLimit);
      return this;
    }

    public Builder traceId(long value) {
      if (lastFieldSet < INDEX_TIMESTAMP) {
        timestamp(DEFAULT_TIMESTAMP);
      }
      assert lastFieldSet == INDEX_TRACE_ID - 1;
      int newLimit = limit() + FIELD_SIZE_TRACE_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_TRACE_ID;
      limit(newLimit);
      return this;
    }

    public Builder authorization(long value) {
      if (lastFieldSet < INDEX_TRACE_ID) {
        traceId(DEFAULT_TRACE_ID);
      }
      assert lastFieldSet == INDEX_AUTHORIZATION - 1;
      int newLimit = limit() + FIELD_SIZE_AUTHORIZATION;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_AUTHORIZATION;
      limit(newLimit);
      return this;
    }

    public Builder budgetId(long value) {
      if (lastFieldSet < INDEX_AUTHORIZATION) {
        authorization(DEFAULT_AUTHORIZATION);
      }
      assert lastFieldSet == INDEX_BUDGET_ID - 1;
      int newLimit = limit() + FIELD_SIZE_BUDGET_ID;
      checkLimit(newLimit, maxLimit());
      buffer().putLong(limit(), value);
      lastFieldSet = INDEX_BUDGET_ID;
      limit(newLimit);
      return this;
    }

    public Builder credit(int value) {
      assert lastFieldSet == INDEX_CREDIT - 1;
      int newLimit = limit() + FIELD_SIZE_CREDIT;
      checkLimit(newLimit, maxLimit());
      buffer().putInt(limit(), value);
      lastFieldSet = INDEX_CREDIT;
      limit(newLimit);
      return this;
    }

    public Builder padding(int value) {
      assert lastFieldSet == INDEX_PADDING - 1;
      int newLimit = limit() + FIELD_SIZE_PADDING;
      checkLimit(newLimit, maxLimit());
      buffer().putInt(limit(), value);
      lastFieldSet = INDEX_PADDING;
      limit(newLimit);
      return this;
    }

    public Builder capabilities(int value) {
      if (value < 0) {
        throw new IllegalArgumentException(String.format("Value %d too low for field \"capabilities\"", value));
      }
      if (value > 0XFF) {
        throw new IllegalArgumentException(String.format("Value %d too high for field \"capabilities\"", value));
      }
      assert lastFieldSet == INDEX_CAPABILITIES - 1;
      int newLimit = limit() + FIELD_SIZE_CAPABILITIES;
      checkLimit(newLimit, maxLimit());
      buffer().putByte(limit(), (byte)(value & 0xFF));
      lastFieldSet = INDEX_CAPABILITIES;
      limit(newLimit);
      return this;
    }

    @Override
    public Builder wrap(MutableDirectBuffer buffer, int offset, int maxLimit) {
      super.wrap(buffer, offset, maxLimit);
      lastFieldSet = -1;
      super.wrap(buffer, offset, maxLimit);
      limit(offset);
      return this;
    }

    @Override
    public Builder rewrap() {
      super.rewrap();
      return this;
    }

    @Override
    public WindowFW build() {
      if (lastFieldSet < INDEX_CAPABILITIES) {
        capabilities(DEFAULT_CAPABILITIES);
      }
      assert lastFieldSet == FIELD_COUNT - 1;
      lastFieldSet = -1;
      return super.build();
    }
  }
}
